import bleak.backends.characteristic
import bleak.backends.client


async def get_characteristic(
    gatt_client: bleak.backends.client.BaseBleakClient,
    char_uuid,
) -> bleak.backends.characteristic.BleakGATTCharacteristic:
    services = await gatt_client.get_services()
    service = services.get_service(char_uuid.SERVICE_UUID.value)
    return service.get_characteristic(char_uuid.value)


async def write_to_characteristic(
    gatt_client: bleak.backends.client.BaseBleakClient,
    uuid,
    data: bytes,
) -> None:
    characteristic = await get_characteristic(gatt_client, uuid)
    await gatt_client.write_gatt_char(characteristic, data, response=True)


async def read_from_characteristic(
    gatt_client: bleak.backends.client.BaseBleakClient,
    uuid,
) -> bytes:
    characteristic = await get_characteristic(gatt_client, uuid)
    return await gatt_client.read_gatt_char(characteristic)



import enum
import math

from cryptography.hazmat.primitives.ciphers import algorithms
from cryptography.hazmat.primitives.ciphers import Cipher
from cryptography.hazmat.primitives.ciphers import modes


class SX3Profile:
    def __init__(self, key: str) -> None:
        self._cipher = Cipher(algorithms.AES(bytes.fromhex(key)), modes.ECB())

    def build_authentication_payload(self, nonce: bytes) -> bytes:
        encryptor = self._cipher.encryptor()

        data = bytearray(16)
        data[0:2] = nonce
        data = bytearray(encryptor.update(data) + encryptor.finalize())

        # unknown why we need to append these 4 bytes _after_ encryption
        data.extend([0, 0, 0, 2])

        return bytes(data)

    def decrypt_payload(self, data: bytes) -> bytes:
        decryptor = self._cipher.decryptor()
        return decryptor.update(data) + decryptor.finalize()

    def build_encrypted_payload(self, nonce: bytes, data: bytes) -> bytes:
        encryptor = self._cipher.encryptor()

        payload = bytearray(16)
        payload[0:2] = nonce
        payload[2:] = data

        # Pad to the nearest cipher 16 byte block size
        for _ in range(math.ceil(len(payload) / 16) * 16 - len(payload)):
            payload.append(0)

        return bytes(encryptor.update(payload) + encryptor.finalize())

    class Security(enum.Enum):

        SERVICE_UUID = "6acc5500-e631-4069-944d-b8ca7598ad50"

        CHALLENGE = "6acc5501-e631-4069-944d-b8ca7598ad50"
        KEY_INDEX = "6acc5502-e631-4069-944d-b8ca7598ad50"
        BACKUP_CODE = "6acc5503-e631-4069-944d-b8ca7598ad50"
        BIKE_MESSAGE = "6acc5505-e631-4069-944d-b8ca7598ad50"

    class Defense(enum.Enum):

        SERVICE_UUID = "6acc5520-e631-4069-944d-b8ca7598ad50"

        LOCK_STATE = "6acc5521-e631-4069-944d-b8ca7598ad50"
        UNLOCK_REQUEST = "6acc5522-e631-4069-944d-b8ca7598ad50"
        ALARM_STATE = "6acc5523-e631-4069-944d-b8ca7598ad50"
        ALARM_MODE = "6acc5524-e631-4069-944d-b8ca7598ad50"

    class Movement(enum.Enum):

        SERVICE_UUID = "6acc5530-e631-4069-944d-b8ca7598ad50"

        DISTANCE = "6acc5531-e631-4069-944d-b8ca7598ad50"
        SPEED = "6acc5532-e631-4069-944d-b8ca7598ad50"
        UNIT_SYSTEM = "6acc5533-e631-4069-944d-b8ca7598ad50"
        POWER_LEVEL = "6acc5534-e631-4069-944d-b8ca7598ad50"
        SPEED_LIMIT = "6acc5535-e631-4069-944d-b8ca7598ad50"
        E_SHIFTER_GEAR = "6acc5536-e631-4069-944d-b8ca7598ad50"
        E_SHIFTIG_POINTS = "6acc5537-e631-4069-944d-b8ca7598ad50"
        E_SHIFTER_MODE = "6acc5538-e631-4069-944d-b8ca7598ad50"

    class BikeInfo(enum.Enum):

        SERVICE_UUID = "6acc5540-e631-4069-944d-b8ca7598ad50"

        MOTOR_BATTERY_LEVEL = "6acc5541-e631-4069-944d-b8ca7598ad50"
        MOTOR_BATTERY_STATE = "6acc5542-e631-4069-944d-b8ca7598ad50"
        MODULE_BATTERY_LEVEL = "6acc5543-e631-4069-944d-b8ca7598ad50"
        MODULE_BATTERY_STATE = "6acc5544-e631-4069-944d-b8ca7598ad50"
        BIKE_FIRMWARE_VERSION = "6acc554a-e631-4069-944d-b8ca7598ad50"
        BLE_CHIP_FIRMWARE_VERSION = "6acc554b-e631-4069-944d-b8ca7598ad50"
        CONTROLLER_FIRMWARE_VERSION = "6acc554c-e631-4069-944d-b8ca7598ad50"
        PCBA_HARDWARE_VERSION = "6acc554d-e631-4069-944d-b8ca7598ad50"
        GSM_FIRMWARE_VERSION = "6acc554e-e631-4069-944d-b8ca7598ad50"
        E_SHIFTER_FIRMWARE_VERSION = "6acc554f-e631-4069-944d-b8ca7598ad50"
        BATTERY_FIRMWARE_VERSION = "6acc5550-e631-4069-944d-b8ca7598ad50"

        # data returned seems to be firmware version info?
        _UNKNOWN = "6acc5551-e631-4069-944d-b8ca7598ad50"

        FRAME_NUMBER = "6acc5552-e631-4069-944d-b8ca7598ad50"

    class BikeState(enum.Enum):

        SERVICE_UUID = "6acc5560-e631-4069-944d-b8ca7598ad50"

        MODULE_MODE = "6acc5561-e631-4069-944d-b8ca7598ad50"
        MODULE_STATE = "6acc5562-e631-4069-944d-b8ca7598ad50"
        ERRORS = "6acc5563-e631-4069-944d-b8ca7598ad50"
        WHEEL_SIZE = "6acc5564-e631-4069-944d-b8ca7598ad50"
        CLOCK = "6acc5567-e631-4069-944d-b8ca7598ad50"

    class Sound(enum.Enum):

        SERVICE_UUID = "6acc5570-e631-4069-944d-b8ca7598ad50"

        PLAY_SOUND = "6acc5571-e631-4069-944d-b8ca7598ad50"
        SOUND_VOLUME = "6acc5572-e631-4069-944d-b8ca7598ad50"
        BELL_SOUND = "6acc5574-e631-4069-944d-b8ca7598ad50"

    class Light(enum.Enum):

        SERVICE_UUID = "6acc5580-e631-4069-944d-b8ca7598ad50"

        LIGHT_MODE = "6acc5581-e631-4069-944d-b8ca7598ad50"
        SENSOR = "6acc5584-e631-4069-944d-b8ca7598ad50"



from enum import Enum

import bleak.backends.client

from pymoof.profiles.sx3 import SX3Profile
from pymoof.util import bleak_utils


class BellTone(Enum):

    BOAT = 0x18
    PARTY = 0x17
    BELL = 0x16


class LockState(Enum):

    UNLOCKED = 0x00
    LOCKED = 0x01
    AWAITING_UNLOCK = 0x02


class Sound(Enum):

    SCROLLING_TONE = 0x1
    BEEP_NEGATIVE = 0x2
    BEEP_POSITIVE = 0x3
    UNLOCK_COUNTDOWN = 0x4
    PAIRING = 0x5
    ENTER_BACKUP_CODE_MODE = 0x6
    RESET_COUNTDOWN = 0x7
    PAIRING_SUCCESSFUL = 0x8
    PAIRING_FAILED = 0x9
    HORN_1 = 0xA
    HORN_URGENT = 0xB
    LOCK = 0xC
    UNLOCK = 0xD
    ALARM_STAGE_ONE = 0xE
    ALARM_STAGE_TWO = 0xF
    SYSTEM_STARTUP = 0x10
    SYSTEM_SHUTDOWN = 0x11
    CHARGING = 0x12
    DIAGNOSE = 0x13
    FIRMWARE_DOWNLOAD = 0x14
    FIRMWARE_FAILED = 0x15
    HORN_2 = 0x16
    HORN_3 = 0x17
    HORN_4 = 0x17
    FIRMWARE_SUCCESSFUL = 0x18
    NOISE = 0x19
    UNPAIRING = 0x1A
    FM_DISABLE = 0x1B
    FM_ENABLE = 0x1C
    FM_NOISE = 0x1D


class SX3Client:
    """Collection of functions to interact with a Vanmoof SX3 over bluetooth"""

    def __init__(
        self,
        bleak_client: bleak.backends.client.BaseBleakClient,
        key: str,
    ) -> None:
        self._gatt_client = bleak_client
        self._bike_profile = SX3Profile(key)

    async def _get_nonce(self) -> bytes:
        return await self._read(
            self._bike_profile.Security.CHALLENGE,
            needs_decryption=False,
        )

    async def _read(self, characteristic_uuid, needs_decryption: bool = True) -> bytes:
        result = await bleak_utils.read_from_characteristic(
            self._gatt_client,
            characteristic_uuid,
        )

        if needs_decryption:
            result = self._bike_profile.decrypt_payload(result)

        return result

    async def _write(self, characteristic_uuid, data: bytes) -> None:
        nonce = await self._get_nonce()
        payload = self._bike_profile.build_encrypted_payload(nonce, data)

        await bleak_utils.write_to_characteristic(
            self._gatt_client,
            characteristic_uuid,
            payload,
        )

    async def authenticate(self) -> None:
        nonce = await self._get_nonce()
        payload = self._bike_profile.build_authentication_payload(nonce)

        await bleak_utils.write_to_characteristic(
            self._gatt_client,
            self._bike_profile.Security.KEY_INDEX,
            payload,
        )

    async def set_bell_tone(self, bell_tone: BellTone) -> None:
        await self._write(
            self._bike_profile.Sound.BELL_SOUND,
            [bell_tone.value],
        )

    async def set_lock_state(self, state: LockState) -> None:
        await self._write(
            self._bike_profile.Defense.LOCK_STATE,
            [state.value],
        )

    async def set_power_level(self, level: int) -> None:
        assert 0 <= level <= 5

        await self._write(
            self._bike_profile.Movement.POWER_LEVEL,
            [level, 0x1],
        )

    async def play_sound(self, sound: Sound, count: int = 1):
        assert 0 < count
        await self._write(
            self._bike_profile.Sound.PLAY_SOUND,
            [sound.value, count],
        )

    async def get_battery_level(self) -> int:
        result = await self._read(
            self._bike_profile.BikeInfo.MOTOR_BATTERY_LEVEL,
        )

        return int(result[0])

    async def get_lock_state(self) -> LockState:
        result = await self._read(
            self._bike_profile.Defense.LOCK_STATE,
        )

        return LockState(result[0])

    async def get_distance_travelled(self) -> float:
        # Returns kilometers, stored as hectometers
        result = await self._read(
            self._bike_profile.Movement.DISTANCE,
        )
        return int.from_bytes(result, "little") / 10

    async def get_power_level(self) -> int:
        result = await self._read(
            self._bike_profile.Movement.POWER_LEVEL,
        )
        return int.from_bytes(result, "little")

    async def get_frame_number(self) -> str:
        result = await self._read(
            self._bike_profile.BikeInfo.FRAME_NUMBER,
            needs_decryption=False,
        )

        return result.decode("ascii")

    async def get_sound_volume(self) -> bytes:
        result = await self._read(
            self._bike_profile.Sound.SOUND_VOLUME,
        )
        return int.from_bytes(result, "little")

    async def get_speed(self) -> bytes:
        result = await self._read(
            self._bike_profile.Movement.SPEED,
        )
        return int.from_bytes(result, "little")



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pymoof', 'pymoof.clients', 'pymoof.profiles', 'pymoof.tools', 'pymoof.util']

package_data = \
{'': ['*']}

install_requires = \
['bleak>=0.14.2,<0.15.0',
 'coverage>=6.3,<7.0',
 'cryptography>=36.0.1,<37.0.0',
 'pytest-asyncio>=0.17.2,<0.18.0',
 'pytest-cov>=3.0.0,<4.0.0',
 'requests>=2.27.1,<3.0.0']

setup_kwargs = {
    'name': 'pymoof',
    'version': '0.0.3',
    'description': 'Connect to your Vanomof S3/X3 bike',
    'long_description': '# pymoof\nConnect to your Vanmoof S3 and X3 through bluetooth.\n\n## Installation\nInstall python 3.6+, then use pip to install pymoof.\n`pip install pymoof`\n\n## Usage\n```python\nfrom pymoof.clients.sx3 import SX3Client\n\nclient = SX3Client(bleak_client, encryption_key)\nclient.authenticate()\n```\nYou must have an instantiated bleak client that is connected to the bike. See `pymoof.tools.discover_bike` to determine which device is your bike and `pymoof.tools.retrieve_encryption_key` to connect to Vanmoof servers to get your encryption key.\n\nSee `example.py` for more info on useage.\n\n## Contributing\nThis project uses Poetry for package and dependency management. It also uses tox for test running and pre-commit for running linters.\n',
    'author': 'Henri Bai',
    'author_email': 'quantsini@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/quantsini/pymoof',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7',
}


setup(**setup_kwargs)



import asyncio

from bleak import BleakScanner


async def query():
    devices = await BleakScanner.discover(
        service_uuids=[
            "6acc5540-e631-4069-944d-b8ca7598ad50",
            "8e7f1a50-087a-44c9-b292-a2c628fdd9aa",
            "6acb5520-e631-4069-944d-b8ca7598ad50",
        ],
    )
    for device in devices:
        if "6acc5540-e631-4069-944d-b8ca7598ad50" in device.metadata["uuids"]:
            print("Found SX3:", device.address)
            return device
        if "8e7f1a50-087a-44c9-b292-a2c628fdd9aa" in device.metadata["uuids"]:
            print("Found SX1/SX2:", device.address, "but it's not supported")
            raise Exception()
        if "6acb5520-e631-4069-944d-b8ca7598ad50" in device.metadata["uuids"]:
            print("Found Smart SX1:", device.address, "but it's not supported")
            raise Exception()
    print("No Vanmoof bikes found")


if __name__ == "__main__":
    asyncio.run(query())



import base64
import getpass

import requests


def query():
    API_URL = "https://my.vanmoof.com/api/v8"
    API_KEY = "fcb38d47-f14b-30cf-843b-26283f6a5819"

    username = input("Username: ")
    password = getpass.getpass()

    headers = {
        "Api-Key": API_KEY,
        "Authorization": "Basic "
        + base64.b64encode((username + ":" + password).encode()).decode("ascii"),
    }

    result = requests.post(API_URL + "/authenticate", headers=headers)
    result = result.json()

    if "error" in result:
        raise Exception("error", result)

    token = result["token"]

    headers = {
        "Api-Key": API_KEY,
        "Authorization": "Bearer " + token,
    }

    result = requests.get(
        API_URL + "/getCustomerData",
        headers=headers,
        params={"includeBikeDetails": ""},
    )
    result = result.json()

    # Only get the first bike's encryption key
    return result["data"]["bikeDetails"][0]["key"]["encryptionKey"]


if __name__ == "__main__":
    print(query())
